/*	$OpenBSD: stacktrace.h,v 1.4 2023/04/26 16:53:59 claudio Exp $	*/

/*
 * Copyright (c) 2017 Visa Hankala
 *
 * Permission to use, copy, modify, and/or distribute this software for any
 * purpose with or without fee is hereby granted, provided that the above
 * copyright notice and this permission notice appear in all copies.
 *
 * THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES
 * WITH REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR
 * ANY SPECIAL, DIRECT, INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN
 * ACTION OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF
 * OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 */

#ifndef _SYS_STACKTRACE_H_
#define _SYS_STACKTRACE_H_

#define STACKTRACE_MAX	19

struct stacktrace {
	unsigned int	st_count;
	unsigned long	st_pc[STACKTRACE_MAX];
};

#ifdef _KERNEL
void	stacktrace_print(struct stacktrace *, int (*)(const char *, ...));
void	stacktrace_save_at(struct stacktrace *, unsigned int);
void	stacktrace_save_utrace(struct stacktrace *);

static inline void
stacktrace_save(struct stacktrace *st)
{
	stacktrace_save_at(st, 0);
}
#endif

#endif /* _SYS_STACKTRACE_H_ */
