/* $OpenBSD: ascii.h,v 1.6 2017/05/27 12:00:28 fcambus Exp $ */
/* $NetBSD: ascii.h,v 1.3 1998/06/20 19:11:04 drochner Exp $ */

#define ASCII_NUL	0x00		/* nul */
#define ASCII_BEL	0x07		/* bell */
#define ASCII_BS	0x08		/* backspace */
#define ASCII_HT	0x09		/* horizontal tab */
#define ASCII_LF	0x0a		/* line feed */
#define ASCII_VT	0x0b		/* vertical tab */
#define ASCII_FF	0x0c		/* form feed */
#define ASCII_CR	0x0d		/* carriage return */
#define ASCII_SO	0x0e		/* shift out */
#define ASCII_SI	0x0f		/* shift in */
#define ASCII_CAN	0x18		/* cancel */
#define ASCII_SUB	0x1a		/* substitute */
#define ASCII_ESC	0x1b		/* escape */
